﻿<#
.SYNOPSIS
   Utility functions for interfacing with Content Manager Core Service and Import/Export Service
.DESCRIPTION
    This script is intended to be included (dot-sourced) into other PowerShell scripts.
    Note: this script expects the following variables to be defined/set by the invoking script: 
        $cmsUrl
        $cmsAuth
        $cmsUserName (optional)
        $cmsUserPassword (optional)

    Based on:
    http://www.indivirtual.nl/blog/sdl-tridions-importexport-api-end-content-porter/
    http://www.indivirtual.nl/blog/set-rights-permissions-using-sdl-tridion-core-service-api/
#>

function Get-TempFolder($folderName) {
    $tempFolder = Join-Path $env:ALLUSERSPROFILE "$folderName\"
    if (!(Test-Path $tempFolder)) {
        New-Item -ItemType Directory -Path $tempFolder | Out-Null
    }
    Write-Verbose "Temp folder is '$tempFolder'"
    return $tempFolder
}

function Initialize-ImportExport ($importExportFolder, $tempFolder) {
    Write-Verbose "Initializing Import/Export using folder '$importExportFolder'"

    if ($importExportFolder.StartsWith("\\") -or !((New-Object System.IO.DriveInfo (New-Object System.IO.FileInfo $importExportFolder).Directory.Root.FullName).DriveType -eq "Fixed")) {
        # Copy all DLLs to a local accessible location, since we cannot load DLLs from a network location
        if (!(Test-Path $tempFolder\Tridion.ContentManager.ImportExport.*)) {
            #Copy-Item works weird if destination folder does not exist
            if (!(Test-Path $tempFolder)) {
                Write-Verbose "Creating temp folder ..."
                New-Item -ItemType Directory -Path $tempFolder | Out-Null
            }
            Write-Verbose "Copying ImportExport assemblies to '$tempFolder' ..."
            Copy-Item "$($importExportFolder)\*" $tempFolder -Recurse -Force
            Write-Verbose "Done"
        }
        $importExportFolder = $tempFolder
    }

    Add-Type -assemblyName mscorlib
    Add-Type -assemblyName System.ServiceModel
    Add-Type -Path "$($importExportFolder)\ChilkatDotNet4.dll"
    Add-Type -Path "$($importExportFolder)\Tridion.Common.dll"
    Add-Type -Path "$($importExportFolder)\Tridion.ContentManager.CoreService.Client.dll"
    Add-Type -Path "$($importExportFolder)\Tridion.ContentManager.ImportExport.Common.dll"
    Add-Type -Path "$($importExportFolder)\Tridion.ContentManager.ImportExport.Client.dll"

    Write-Verbose "Done."
}

function Set-ClientCredentials($wcfClient) {
    switch ($cmsAuth) {
        "Windows" {
            if ($cmsUserName) {
                Write-Verbose "Using Windows authentication with CMS user name: $cmsUserName"
                $wcfClient.ClientCredentials.Windows.ClientCredential = New-Object System.Net.NetworkCredential $cmsUserName, $cmsUserPassword
            }
            else {
                Write-Verbose "Using Windows authentication with the current Windows user's credentials (no CMS user specified explicitly)." 
                $wcfClient.ClientCredentials.Windows.ClientCredential = [System.Net.CredentialCache]::DefaultNetworkCredentials;
            }
        }
        "Basic" {
            if (!$cmsUserName -or !$cmsUserPassword) {
                throw "CMS user name and password must be specified for Basic authentication."
            }
            Write-Verbose "Using Basic authentication with CMS user name: $cmsUserName"
            $wcfClient.ClientCredentials.UserName.UserName = $cmsUserName
            $wcfClient.ClientCredentials.UserName.Password = $cmsUserPassword
        }
    }

}

function Get-ImportExportServiceClient {
    param(
        [parameter(Mandatory=$false)]
        [ValidateSet("Service","Upload","Download")]
        [string]$type="Service"
    )
    Write-Verbose "Getting Import/Export Service Client with type '$type' for CMS URL '$cmsUrl' ..."

	$binding = New-Object System.ServiceModel.BasicHttpBinding
	$binding.MaxBufferPoolSize = [int]::MaxValue
	$binding.MaxReceivedMessageSize = [int]::MaxValue
	$binding.ReaderQuotas.MaxArrayLength = [int]::MaxValue
	$binding.ReaderQuotas.MaxBytesPerRead = [int]::MaxValue
	$binding.ReaderQuotas.MaxStringContentLength = [int]::MaxValue
	$binding.ReaderQuotas.MaxNameTableCharCount = [int]::MaxValue


	switch($type)
	{
		"Service" {
            if ($cmsUrl.StartsWith("https")) { $binding.Security.Mode = "Transport" }
            else { $binding.Security.Mode = "TransportCredentialOnly" }
			$binding.Security.Transport.ClientCredentialType = $cmsAuth
			$endpoint = New-Object System.ServiceModel.EndpointAddress ($cmsUrl + "webservices/ImportExportService2013.svc/basicHttp")
			$client = New-Object Tridion.ContentManager.ImportExport.Client.ImportExportServiceClient $binding,$endpoint
		}
 
		"Download" {
            if ($cmsUrl.StartsWith("https")) { $binding.Security.Mode = "Transport" }
            else { $binding.Security.Mode = "TransportCredentialOnly" }
			$binding.Security.Transport.ClientCredentialType = $cmsAuth
			$binding.TransferMode = "StreamedResponse"
			$binding.MessageEncoding = "Mtom"
			$endpoint = New-Object System.ServiceModel.EndpointAddress ($cmsUrl + "webservices/ImportExportService2013.svc/streamDownload_basicHttp")	
			$client = New-Object Tridion.ContentManager.ImportExport.Client.ImportExportStreamDownloadClient $binding,$endpoint
		}
 
		"Upload" {
            if ($cmsUrl.StartsWith("https")) { $binding.Security.Mode = "Transport" }
            else { $binding.Security.Mode = "None" }
			$binding.TransferMode = "StreamedRequest"
			$binding.MessageEncoding = "Mtom"
			$endpoint = New-Object System.ServiceModel.EndpointAddress ($cmsUrl + "webservices/ImportExportService2013.svc/streamUpload_basicHttp")
			$client = New-Object Tridion.ContentManager.ImportExport.Client.ImportExportStreamUploadClient $binding,$endpoint
		}
	}

    Set-ClientCredentials($client)

    Write-Verbose $client.Endpoint.Address
    return $client
}

function Get-CoreServiceClient {
    param(
        [parameter(Mandatory=$false)]
        [ValidateSet("Service","Upload","Download")]
        [string]$type="Service"
    )        
    Write-Verbose "Getting Core Service Client with type '$type' for CMS URL '$cmsUrl' ..."

	$binding = New-Object System.ServiceModel.WSHttpBinding
	$binding.MaxBufferPoolSize = [int]::MaxValue
	$binding.MaxReceivedMessageSize = [int]::MaxValue
	$binding.ReaderQuotas.MaxArrayLength = [int]::MaxValue
	$binding.ReaderQuotas.MaxBytesPerRead = [int]::MaxValue
	$binding.ReaderQuotas.MaxStringContentLength = [int]::MaxValue
	$binding.ReaderQuotas.MaxNameTableCharCount = [int]::MaxValue

	switch($type)
	{
		"Service" {
            if ($cmsUrl.StartsWith("https")) { $binding.Security.Mode = "TransportWithMessageCredential" }
            else { $binding.Security.Mode = "Message" }
            if ($cmsAuth -eq "Basic") { $binding.Security.Message.ClientCredentialType = "UserName" }
            else { $binding.Security.Message.ClientCredentialType = $cmsAuth }
			$endpoint = New-Object System.ServiceModel.EndpointAddress ($cmsUrl + "webservices/CoreService2013.svc/wsHttp")
			$client = New-Object Tridion.ContentManager.CoreService.Client.SessionAwareCoreServiceClient $binding,$endpoint
        }
	}

    Set-ClientCredentials($client)

    Write-Verbose $client.Endpoint.Address

    if ($type -eq "Service")
    {
        $cmsUser = $client.GetCurrentUser()
        $cmsUserTitle = $cmsUser.Title
        $cmsUserDescription = $cmsUser.Description
        Write-Verbose "Connected to Core Service as user '$cmsUserTitle' ($cmsUserDescription)" 
    }

    return $client;
}

function Get-Trustee($group, $coreServiceClient) {
    # load trustee from group name
    $groupsFilter = New-Object Tridion.ContentManager.CoreService.Client.groupsFilterData
    $list = $coreServiceClient.GetSystemWideList($groupsFilter)
    foreach ($trustee in $list) {
        if ($trustee.Title -match $group) {
            return $trustee
        }
    }
    return $null
}

function Set-RightsAndPermissions($trustee, $orgItem, $rights, $permissions, $coreServiceClient) {
    Write-Verbose "Updating $($orgItem.LocationInfo.Webdavurl) for $($trustee.Id) ..."

    # need to strip out any existing entries for this trustee
    $entries = $orgItem.AccessControlList.AccessControlEntries | ? {$_.Trustee.IdRef -ne $trustee.Id}
    $link = New-Object Tridion.ContentManager.CoreService.Client.LinkToTrusteeData
    $link.IdRef = $trustee.Id
 
    $ace = New-Object Tridion.ContentManager.CoreService.Client.AccessControlEntryData
    $ace.Trustee = $link
    if ($rights -ne $null) {$ace.AllowedRights = $rights}
    if ($permissions -ne $null) {
        $ace.AllowedPermissions = $permissions
        $orgItem.IsPermissionsInheritanceRoot = $true
    }
    $entries += $ace
    $orgItem.AccessControlList.AccessControlEntries = $entries
    $coreServiceClient.Save($orgItem, $null)
    
    Write-Verbose "Done"
}

function Set-Rights($webdavUrl, $group, $rights, $coreServiceClient) {
    Write-Output "Setting '$rights' for '$group' in $webdavUrl ..."    

    $trustee = Get-Trustee $group $coreServiceClient
    try {
        $item = $coreServiceClient.Read($webdavUrl, $defaultReadOptions)
        Set-RightsAndPermissions $trustee $item $rights $null $coreServiceClient
    } 
    catch {
        Write-Warning "Skipped, $webdavUrl does not exist"
    }
}

function Set-Permissions($webdavUrl, $group, $permissions, $coreServiceClient) {
    Write-Output "Setting '$permissions' for '$group' in $webdavUrl ..."

    $trustee = Get-Trustee $group $coreServiceClient
    try {
        $item = $coreServiceClient.Read($webdavUrl, $defaultReadOptions)
        Set-RightsAndPermissions $trustee $item $null $permissions $coreServiceClient
    } 
    catch {
        Write-Warning "Skipped, $webdavUrl does not exist"
    }
}

function Invoke-UploadPackageFromFile($packageLocation) {
    Write-Verbose "Uploading package from file ..."
    Write-Verbose "PackageLocation is '$packageLocation'"
	$uploadService = Get-ImportExportServiceClient -type Upload
	try {
		$packageStream = [IO.File]::OpenRead($packageLocation)
		$uploadService.UploadPackageStream($packageStream)
	}
	finally {
		if ($packageStream -ne $null){
			$packageStream.Dispose()	
		}
		if ($uploadService -ne $null){
			$uploadService.Dispose()	
		}
	}
    Write-Verbose "Done"
}
 
function Wait-ImportExportFinish($serviceClient, $processId) {
    Write-Verbose "Waiting for Import/Export Process '$processId' to finish ..."

    $waited = 0
    $interval = 5
	do {
		$processState = $serviceClient.GetProcessState($processId)
		if ("Finished", "Aborted", "AbortedByUser" -contains $processState) {
			break;
		}

        Start-Sleep -Seconds $interval
        $waited = $waited + $interval
        Write-Verbose "($waited seconds)"
	} while ($true)

    Write-Verbose "Process State is '$processState'"
    return $processState
}

function Invoke-DownloadLog($processId, $path) {
    Write-Verbose "Downloading log for Import/Export Process '$processId' to '$path' ..."
    $downloadClient = Get-ImportExportServiceClient -type Download
    try {
	    $packageStream = $downloadClient.DownloadProcessLogFile($processId, $true)
	    $fileStream = [IO.File]::Create($path)
	    $packageStream.CopyTo($fileStream)
    }	
    finally { 
	    if ($fileStream -ne $null) {
		    $fileStream.Dispose()
	    }
	    if ($packageStream -ne $null) {
		    $packageStream.Dispose()
	    }
    }
    Write-Verbose "Done"
}

function Invoke-Upload($mapping, $packageFullPath, $tempFolder) {
    Write-Output "Uploading package ..."
    Write-Verbose "Package FullPath is '$packageFullPath'"
    Write-Verbose "Temp Folder is '$tempFolder'"

    $filename = (Get-ChildItem $packageFullPath).BaseName
    $extension = (Get-ChildItem $packageFullPath).Extension
    $uploadId = Invoke-UploadPackageFromFile $packageFullPath

    Write-Output "Importing content ..."
    $impexp = Get-ImportExportServiceClient
    $importInstruction = New-Object Tridion.ContentManager.ImportExport.ImportInstruction
    #$importInstruction.LogLevel = "Debug"
    $importInstruction.LogLevel = "Normal"
    $importInstruction.CreateUndoPackage = $true
    $importInstruction.SchemaSynchronizeFlags = 103

    if($mapping -ne $null) {
        $importInstruction.UserMappings = [Tridion.ContentManager.ImportExport.Packaging.Mapping[]]($mapping)
    }
    
    $processId = $impexp.StartImport($uploadId, ($importInstruction))
    $state = Wait-ImportExportFinish $impexp $processId
    Write-Output "Done"

    $importLogFullPath = "$($tempFolder)$($filename)-import.log"
    Write-Verbose "Import log file is '$importLogFullPath'"
    Invoke-DownloadLog $processId $importLogFullPath
    # Output log to console
    Get-Content $importLogFullPath

    Write-Output "Done Upload: $($state)"

    if($state -ne "Finished") {
        throw "Problem importing file $($filename)$($extension) $($impexp.GetProcessInfo($processId).Messages)"
    }
    if ($impexp -ne $null) {
        $impexp.Dispose()
    }
}
