﻿<#
.SYNOPSIS
   Import the JWPlayer Module into CMS
.DESCRIPTION
   Import the JWPlayer Module into CMS
.EXAMPLE
   & .\cms-import.ps1 -cmsUrl "http://localhost:81/" 
.INPUTS
   importType, cmsUrl, masterPublication, sitePublication, rootStructureGroup
.NOTES
   Importing into existing publication by means of mapping a target publication title is currently not supported for rights and permissions
.COMPONENT
   The component this cmdlet belongs to
.ROLE
   The role this cmdlet belongs to
.FUNCTIONALITY
   The functionality that best describes this cmdlet
#>

[CmdletBinding( SupportsShouldProcess=$true, PositionalBinding=$false)]
param (
    [ValidateSet("module-and-permissions", "module-only", "permissions-only")]
    [string]$importType = "module-and-permissions",

    # Enter your cms url
    [Parameter(Mandatory=$true, HelpMessage="URL of the CMS you want to import in")]
    [string]$cmsUrl,

    # If you are importing into existing publications, update these to map to your target publication titles
    [Parameter(Mandatory=$false, HelpMessage="Title to use for the DXA Master Publication")]
    [string]$masterPublication = "100 Master",

    # If you are importing into existing publications, update these to map to your target publication titles
    [Parameter(Mandatory=$false, HelpMessage="Title to use for the DXA Example Site Publication")]
    [string]$sitePublication = "400 Example Site",

    # If you are importing into existing publications, update these to map to your root folder and structure group title
    [Parameter(Mandatory=$false, HelpMessage="Title to use for the Root Structure Group")]
    [string]$rootStructureGroup = "Home",

    # By default, the current Windows user's credentials are used, but it is possible to specify alternative credentials:
    [Parameter(Mandatory=$false, HelpMessage="CMS User name")]
    [string]$cmsUserName,
    [Parameter(Mandatory=$false, HelpMessage="CMS User password")]
    [string]$cmsUserPassword,
    [Parameter(Mandatory=$false, HelpMessage="CMS Authentication type")]
    [ValidateSet("Windows", "Basic")]
    [string]$cmsAuth = "Windows"
)

#Include functions from ContentManagerUtils.ps1
$PSScriptDir = Split-Path $MyInvocation.MyCommand.Path
$importExportFolder = Join-Path $PSScriptDir "..\..\ImportExport"
. (Join-Path $importExportFolder "ContentManagerUtils.ps1")

#Terminate script on first occurred exception
$ErrorActionPreference = "Stop"

#Process 'WhatIf' and 'Confirm' options
if (!($pscmdlet.ShouldProcess($cmsUrl, "Import DXA JWPlayer Module"))) { return }

#Initialization
$IsInteractiveMode = !((gwmi -Class Win32_Process -Filter "ProcessID=$PID").commandline -match "-NonInteractive") -and !$NonInteractive


function Add-ComponentToPage($sitePublication, $rootStructureGroup, $componentPath, $templatePath, $pagePath) {
    $pageWebdavUrl = [string]::Format("/webdav/{0}/{1}/{2}.tpg", $sitePublication, $rootStructureGroup, $pagePath)
    $componentWebdavUrl = [string]::Format("/webdav/{0}/Building Blocks/{1}.xml", $sitePublication, $componentPath)
    $templateWebdavUrl = [string]::Format("/webdav/{0}/Building Blocks/{1}.tctcmp", $sitePublication, $templatePath)
    Write-Output "Adding Component '$($componentWebdavUrl)' to Page '$($pageWebdavUrl)' ..."
    Write-Verbose "Publication is '$sitePublication'"
    Write-Verbose "Structure group is '$rootStructureGroup'"
    Write-Verbose "Page Webdav Url is '$pageWebdavUrl'"
    Write-Verbose "Component Webdav Url is '$componentWebdavUrl'"
    Write-Verbose "Template Webdav Url is '$templateWebdavUrl'"
    $page = $coreServiceClient.Read($pageWebdavUrl, $defaultReadOptions)
    $cps = $page.ComponentPresentations
    $component = $coreServiceClient.Read($componentWebdavUrl, $defaultReadOptions)
    $template = $coreServiceClient.Read($templateWebdavUrl, $defaultReadOptions)
    $notAdded = $true
    Write-Verbose "Checking if Component '$($component.Id)' is already added to Page"
    foreach ($c in $cps)
    {
        #CP already exists on page
        if (($c.ComponentTemplate.IdRef -eq $template.Id) -and ($c.Component.IdRef -eq $component.Id))
        {
            Write-Verbose "Existing Component is found: Component ID ='$($component.Id)'; Template ID = '$($template.Id)'"
            $notAdded = $false
        }
    }
    if ($notAdded)
    {
        $cp = New-Object Tridion.ContentManager.CoreService.Client.ComponentPresentationData
        $cp.Component = New-Object Tridion.ContentManager.CoreService.Client.LinkToComponentData
        $cp.Component.IdRef = $component.Id
        $cp.ComponentTemplate = New-Object Tridion.ContentManager.CoreService.Client.LinkToComponentTemplateData
        $cp.ComponentTemplate.IdRef = $template.Id
        $cps += $cp
        $page = $coreServiceClient.CheckOut($page.Id, $false, $defaultReadOptions)
        $page.ComponentPresentations = $cps
        $page = $coreServiceClient.Save($page, $defaultReadOptions)
        $page = $coreServiceClient.CheckIn($page.Id, $true, $null, $defaultReadOptions)
        Write-Output "Done"
    }
    else
    {
        Write-Warning "Component already added to Page. Skipping this step."
    }
}


$distSource = Join-Path (Split-Path $MyInvocation.MyCommand.Path) "\"

$tempFolder = Get-TempFolder "DXA"

#Format data
$distSource = $distSource.TrimEnd("\") + "\"
$cmsUrl = $cmsUrl.TrimEnd("/") + "/"

$importPackageFullPath = Join-Path $distSource "module-JWPlayer.zip"
Write-Verbose "Import Package FullPath is '$importPackageFullPath'"

$permissionsFullPath = Join-Path $distSource "permissions.xml"
Write-Verbose "Permissions file FullPath is '$permissionsFullPath'"

Initialize-ImportExport $importExportFolder $tempFolder

# Create core service client and default read options
$coreServiceClient = Get-CoreServiceClient "Service"
$defaultReadOptions = New-Object Tridion.ContentManager.CoreService.Client.ReadOptions

# Mappings - only edit these if you really know what you are doing!
$detailedMapping = (
    (New-Object Tridion.ContentManager.ImportExport.Packaging.V2013.Mapping2013("Publication", "/webdav/100 Master",("/webdav/" + $masterPublication))),
    (New-Object Tridion.ContentManager.ImportExport.Packaging.V2013.Mapping2013("StructureGroup", "/webdav/100 Master/Home",("/webdav/" + $masterPublication + "/" + $rootStructureGroup))),
    (New-Object Tridion.ContentManager.ImportExport.Packaging.V2013.Mapping2013("Publication", "/webdav/400 Example Site",("/webdav/" + $sitePublication))),
    (New-Object Tridion.ContentManager.ImportExport.Packaging.V2013.Mapping2013("StructureGroup", "/webdav/400 Example Site/Home",("/webdav/" + $sitePublication + "/" + $rootStructureGroup)))
)

if ($importType -ne "permissions-only")
{
    Invoke-Upload $detailedMapping $importPackageFullPath $tempFolder
    Add-ComponentToPage $sitePublication $rootStructureGroup "Modules Content/JWPlayer/Video" "Modules/JWPlayer/Editor/Templates/JWPlayer"  "000 Home" 
}

#   NOTE - this should be executed last after importing all modules and does not work for mapped publications
if ($importType -ne "module-only")
{
    # for rights and permissions import we do not map publications (this doesn't work - need to change the permissions.xml file manually to map to the right publications)
    [xml]$xml = Get-Content $permissionsFullPath
    foreach ($publication in $xml.export.publication) {
        $path = $publication.path
        foreach ($entry in $publication.rights) {
            $group = $entry.group
            $rights = $entry.innerText
            Set-Rights $path $group $rights $coreServiceClient
        }
    }
    foreach ($orgItem in $xml.export.organizationalItem) {
        $path = $orgItem.path
        foreach ($entry in $orgItem.permissions) {
            $group = $entry.group
            $permissions = $entry.innerText
            Set-Permissions $path $group $permissions $coreServiceClient
        }
    }
}

$coreServiceClient.Dispose()
